/***********************************************************************
 *
 * PROJECT: PalmXQ
 * FILE:    mainform.c
 * AUTHOR:  Cheok Yan Cheng
 *
 * DESCRIPTION: Main form for PalmXQ.
 *
 * LAST REVISION:   24/02/2002
 *
 ***********************************************************************/

#include <PalmOS.h>
#include "palmxqrsc.h"
#include "data.h"
#include "protos.h"
#include "defs.h"

static Boolean MainOnMenu(UInt16 menuID);
static Boolean MainOnPenDown(EventPtr event);
static void MainListDrawFun(UInt16 itemNum, RectangleType *bounds,
                        Char **itemsText);
static Boolean MainOnFrmOpen(EventPtr event);
static Boolean MainOnLstSelect(ListType* list, Int16 selectedIndex);
static void ShowGameStatus(void);
static void MainAddListItem(ListType *list, Char *item, Boolean update);
static Boolean MainOnNewGame(void);
static Boolean MainOnComputerLight(void);
static Boolean MainOnComputerDark(void);
static Boolean MainOnComputerStop(void);
static Boolean MainOnComputerDepth(void);
static Boolean MainOnComputerInfo(void);
static Boolean MainOnFlipBoard(void);
static Boolean MainOnSetupBoard(void);
static Boolean MainOnOptionsAboutPalmXQ(void);
static Boolean MainOnOpenGame(void);
static Boolean MainOnSaveGame(void);
static Boolean MainOnRewind(void);
static void ComputerBusyMessage(void);
static void ComputerMakeMove(Int16 side);

static Boolean Selected;    // Has pen selected any pieces?

static Boolean MainOnLstSelect(ListType* list, Int16 selectedIndex)
{
    Boolean handled = false;
    FormType *form;
    Int16 i;

    form = FrmGetActiveForm();
    selectedIndex = LstGetSelection(list);

    if(selectedIndex == noListSelection)
    {
    }
    else
    {
        // Forward move
        if(HPly > selectedIndex)
        {
            Selected = false;

            for(i = HPly; i > selectedIndex; i--)
            {
                UnMakeMove();

                /************************************
                * The below code has a serious bug. It will fill
                * up all the space of the event queue before our main event
                * handle routine can empty it.
                *
                * FrmUpdateForm(FrmGetFormId(FrmGetActiveForm()),
                *                 frmRedrawnBoardCode);
                *************************************/
            }
            DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);

            if(HPly != 0)
            {
                DrawPieceBound(form, MainBoardGadget,
                HistDat[HPly - 1].m.b.to, IsFlip);
            }

            Ply = 0;
            Gen();

        }

        // Backward move
        else if(HPly  < selectedIndex) 
        {
            Selected = false;

            for(i = HPly; i < selectedIndex; i++)
            {
                MakeMove(HistDat[HPly].m);

                /************************************
                * The below code has a serious bug. It will fill
                * up all the space of the event queue before our main event
                * handle routine can empty it.
                *
                * FrmUpdateForm(FrmGetFormId(FrmGetActiveForm()),
                *                 frmRedrawnBoardCode);
                *************************************/
            }

            DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);

            if(HPly != 0)
            {
                DrawPieceBound(form, MainBoardGadget,
                HistDat[HPly - 1].m.b.to, IsFlip);
            }

            Ply = 0;
            Gen();

        }
    }

    // This handled is always false so that the list drawing routine will
    // be handled by the system.
    return handled;
}

static Boolean MainOnMenu(UInt16 menuID)
{
    Boolean    handled = false;
    FormType  *form;

    form = FrmGetActiveForm();

    switch (menuID)
    {
        case MainNewGame:
            handled = MainOnNewGame();
            break;
            
        case MainRewind:
            handled = MainOnRewind();
            break;

/*
        case MainComputerLight:
            handled = MainOnComputerLight();
            break;

        case MainComputerDark:
            handled = MainOnComputerDark();
            break;

        case MainComputerStop:
            handled = MainOnComputerStop();
            break;
*/            

        case MainComputerDepth:
            handled = MainOnComputerDepth();
            break;

        case MainComputerInfo:
            handled = MainOnComputerInfo();
            break;

        case MainFlipBoard:
            handled = MainOnFlipBoard();
            break;

        case MainSetupBoard:
            handled = MainOnSetupBoard();
            break;

        case MainOptionsAboutPalmXQ:
            handled = MainOnOptionsAboutPalmXQ();
            break;
            
        default:
            break;
    }

    return handled;
}

static Boolean MainOnPenDown(EventPtr event)
{
    Boolean handled = false;
    Boolean found;
    Int16 location;
    Int16 i;
    FormType  *form;
    ListType  *list;
    Char wxf[5];
    static MoveT move;

    form = FrmGetActiveForm();
    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, MainMoveList));

    if(ScrnToBrdCoord(form, MainBoardGadget,
                    event->screenX, event->screenY, IsFlip, &location))
    {
        if(Side == Color[location])
        {
            Selected = true;
            move.b.from = location;
            DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);
            DrawPieceBound(form, MainBoardGadget, location, IsFlip);
        }
        else
        {
            if(Selected == true)
            {
                move.b.to = location;

                found = false;
                for(i = 0; i < FirstMove[1]; i++)
                {
                    if(move.u == GenDat[i].m.u)
                    {
                        found = true;
                        break;
                    }
                }

                if(!found || HPly >= HIST_STACK || Fifty >= 100 || Repeat() >= 3 ||
                    !MakeMove(move))
                {
                    FrmAlert(IllegalAlert);
                }
                else
                {
                    // Clean up remaining move list
                    LstSetListChoices(list, NULL, LstGetSelection(list) + 1);
                    LstDrawList(list);

                    // Add new move list
                    BrdCoordToWXF(move.b.from, move.b.to, wxf);
                    MainAddListItem(list, wxf, true);
                    LstSetSelection(list, LstGetNumberOfItems(list) - 1);

                    DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);
                    DrawPieceBound(form, MainBoardGadget, location, IsFlip);

                    Ply = 0;
                    Gen();
                    ShowGameStatus();

                    if(Side == ComputerSide)
                    {
                        ComputerBusyMessage();
                        Think();
                        if(PV[0][0].u != 0)
                        {
							move = PV[0][0];
							MakeMove(move);
                            
							location = move.b.to;							
							
							// Add new move list
                            BrdCoordToWXF(move.b.from, move.b.to, wxf);
                            MainAddListItem(list, wxf, true);
                            LstSetSelection(list, LstGetNumberOfItems(list) - 1);

                            DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);
                            DrawPieceBound(form, MainBoardGadget, location, IsFlip);

                            Ply = 0;
                            Gen();
                            ShowGameStatus();
                        }
                        else
                        {
                            // Erase busy message
                            DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);
                        }
                    }

                    Selected = false;

                }

            }   // if(Selected == true)

        }   //  if(Side == Color[location])

        handled = true;
    }

    return handled;
}

static Boolean MainOnFrmOpen(EventPtr event)
{
    Boolean  handled = false;
    ListType *list;
    FormType  *form;

    form = FrmGetActiveForm();
    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, MainMoveList));

    FrmDrawForm(form);

    InitBoard();

    DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);

    LstSetDrawFunction(list, (ListDrawDataFuncPtr)MainListDrawFun);
    LstSetListChoices(list, NULL, 0);
    MainAddListItem(list, "----", true);
    LstSetSelection(list, 0);

    Selected = false;
    IsFlip = false;
    ComputerSide = EMPTY;
    ComputerDepth = 3;
    PVLength[0] = 0;
    MaxTime = 5;

    handled = true;

    return handled;
}

Boolean MainHandleEvent(EventPtr event)
{
    Boolean  handled = false;
    FormType  *form;
    ListType *list;

    form = FrmGetActiveForm();

    switch (event->eType)
    {
        case frmOpenEvent:
            handled = MainOnFrmOpen(event);
            break;

        case ctlSelectEvent:
            switch (event->data.ctlSelect.controlID)
            {
                default:
                    break;
            }
            break;

        case lstSelectEvent:
            handled = MainOnLstSelect(event->data.lstSelect.pList,
                          LstGetSelection(event->data.lstSelect.pList));
            break;

        case menuEvent:
            handled =
                MainOnMenu(event->data.menu.itemID);
            break;

        case penDownEvent:
            handled =
                MainOnPenDown(event);
            break;
            
        case LOAD_EVENT:
            handled = 
                MainOnOpenGame();
            break;

        case MOVE_EVENT:
            if ( ComputerSide != EMPTY )
                ComputerMakeMove( ComputerSide );
            handled = true;
            break;

        case frmUpdateEvent:
            switch(event->data.frmUpdate.updateCode)
            {
                // passed by setup form
                case frmRedrawUpdateCode:
                    FrmDrawForm(form);
                    DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);
                    
                    handled = true;
                    break;

                case frmRedrawnBoardCode:
                {
                    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, MainMoveList));
                    DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);
                    LstSetListChoices(list, NULL, 1);
                    LstDrawList(list);

                    HPly = 0;
                    Ply = 0;
                    Fifty = 0;

                    Gen();

                    handled = true;
                    break;
                }

                default:
                    break;
            }
            break;

        default:
            break;
    }

    return handled;
}

static void MainListDrawFun(UInt16 itemNum, RectangleType *bounds,
                        Char **itemsText)
{
    WinDrawChars(ListElements[itemNum],
                StrLen(ListElements[itemNum]),
                 bounds->topLeft.x,
                 bounds->topLeft.y);
}

/***********************************************************************
 *
 * FUNCTION:    ShowGameStatus
 *
 * DESCRIPTION: Display a modal dialog box to show up game status -
 *              checkmate, draw by repeating move or draw by fifty move 
 *
 * PARAMETERS:  NONE.
 *
 * RETURNED:    NONE
 *
 * COMMENT:     Before calling this function, the following two statement
 *              must be execute.
 *
 *              Ply = 0;
 *              Gen();
 *              ShowGameStatus();
 *
 *              Otherwise, the behavious of this function is undetermined.
 *
 *              At this moment, all repeated moves (3 times) will be
 *              considered as draw. (That is not true in the real
 *              world XiangQi rules !)
 *
 ***********************************************************************/
void ShowGameStatus()
{
    Boolean isMate;
    Int16 i;

    isMate = true;
    for(i = 0; i < FirstMove[1]; i++)
    {
        if(MakeMove(GenDat[i].m))
        {
            UnMakeMove();
            isMate = false;
            break;
        }
    }

    if(isMate)
    {
        if(Side == DARK)
        {
            FrmCustomAlert(CheckMateAlert, "Light", NULL, NULL);
        }
        else
        {
            FrmCustomAlert(CheckMateAlert, "Dark", NULL, NULL);
        }
    }

    if(Repeat() >= 3)
    {
        FrmAlert(RepeatedDrawAlert);
    }

    if(Fifty >= 100)
    {
        FrmAlert(FiftyDrawAlert);
    }

    if(HPly >= HIST_STACK)
    {
        FrmAlert(HistoryAlert);
    }
}

/***********************************************************************
 *
 * FUNCTION:    MainAddListItem
 *
 * DESCRIPTION: Add the items to the bottom most of list.
 *
 * PARAMETERS:  list        ListType pointer of the list to be operated.
 *              item        Item to be added (String type).
 *              update      Should list be update?
 *
 * RETURNED:    NONE
 *
 * COMMENT:     The maximum length of the string should be ==
 *              sizeof(ListElement) / sizeof(ListElement[0]. Otherwise,
 *              The string will be truncated (If more then) or garbage
 *              characters will be displayed at end of string (If less then). 
 *
 ***********************************************************************/
static void MainAddListItem(ListType *list, Char *item, Boolean update)
{
    Int16 numOfItems;

    numOfItems = LstGetNumberOfItems(list);

    if(numOfItems >= ( sizeof(ListElements) / sizeof(ListElements[0]) ))
    {
        return;
    }

    StrNCopy(ListElements[numOfItems], item, sizeof(ListElements[0]));
    LstSetListChoices(list, NULL, numOfItems + 1);

    if(update)
    {
        LstDrawList(list);
    }
}

static Boolean MainOnNewGame()
{
    Boolean    handled = false;
    FormType  *form;
    ListType  *list;

    form = FrmGetActiveForm();
    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, MainMoveList));

    InitBoard();

    DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);

    LstSetListChoices(list, NULL, 1);
    LstDrawList(list);
    LstSetSelection(list, 0);

    Selected = false;
    Time = 0;
    Node = 0;

    if ( ComputerSide != EMPTY )
        ComputerMakeMove( ComputerSide );

    handled = true;
    return handled;
}

/*
static Boolean MainOnComputerLight()
{
    Boolean    handled = false;

    ComputerMakeMove(LIGHT);

    handled = true;
    return handled;
}

static Boolean MainOnComputerDark()
{
    Boolean    handled = false;

    ComputerMakeMove(DARK);

    handled = true;
    return handled;
}

static Boolean MainOnComputerStop()
{
    Boolean    handled = false;

    ComputerSide = EMPTY;
    
    handled = true;
    return handled;
}
*/

static Boolean MainOnComputerDepth()
{
    Boolean handled = false;

    FrmPopupForm(LevelForm);

    handled = true;
    return handled;
}

static Boolean MainOnComputerInfo()
{
    Boolean    handled = false;
    Char *comSide;
    Char message1[128];
    Char message2[128];
    Char wxf[6];
    Int16 i;

    comSide = (ComputerSide == EMPTY)? "empty":
            ((ComputerSide == LIGHT)? "light": "dark");

    StrPrintF(message1, "Computer searched %lu nodes in %lu seconds.\n", Node, Time );

    StrPrintF(message2, "Computer PV is ");
    for(i = 0; i < PVLength[0]; i++)
    {
        BrdCoordToWXF(PV[0][i].b.from, PV[0][i].b.to, wxf);
        wxf[4] = '-';
        wxf[5] = '\0';
        StrCat(message2, wxf);
    }

    FrmCustomAlert(InfoAlert, comSide, message1, message2);

    handled = true;
    return handled;
}

static Boolean MainOnRewind()
{
    Boolean    handled = false;
    FormType  *form;
    ListType  *list;

    form = FrmGetActiveForm();
    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, MainMoveList));

    LstSetListChoices(list, NULL, LstGetSelection(list) + 1);
    LstDrawList(list);

    if ( ComputerSide != EMPTY )
        ComputerMakeMove( ComputerSide );
}

static Boolean MainOnFlipBoard()
{
    Boolean    handled = false;
    FormType  *form;

    form = FrmGetActiveForm();

    IsFlip = !IsFlip;

    DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);

    Selected = false;

    handled = true;
    return handled;
}

static Boolean MainOnSetupBoard()
{
    Boolean handled = false;

    FrmPopupForm(SetupForm);

    Selected = false;

    handled = true;
    return handled;
}

static Boolean MainOnOptionsAboutPalmXQ()
{
    FormType *form;

    Boolean handled = false;

    form = FrmInitForm(AboutForm);
    FrmDoDialog(form);
    FrmDeleteForm(form);

    handled = true;
    return handled;
}

static void ComputerBusyMessage()
{
    FormType  *form;

    UInt16 gadgetIndex;
    RectangleType gadgetRect;

    form = FrmGetActiveForm();

    // Get the rectangle bound of gadget
    gadgetIndex = FrmGetObjectIndex(form, MainBoardGadget);
    FrmGetObjectBounds(form, gadgetIndex, &gadgetRect);

    WinDrawChars("Thinking...",
            StrLen("Thinking..."),
            gadgetRect.topLeft.x + gadgetRect.extent.x /2 - 19,
            gadgetRect.topLeft.y + gadgetRect.extent.y /2 - 6);
}

static void ComputerMakeMove(Int16 side)
{
    Int16 location;
    MoveT move;
    FormType  *form;
    ListType  *list;
    Char wxf[5];

    form = FrmGetActiveForm();
    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, MainMoveList));
    LstSetSelection(list, LstGetNumberOfItems(list) - 1);
    MainOnLstSelect(list, LstGetNumberOfItems(list) - 1);

    ComputerSide = side;

    if(HPly >= HIST_STACK || Fifty >= 100 || Repeat() >= 3)
    {
        FrmAlert(IllegalAlert);
    }
    else
    {
        // Clean up remaining move list
//        LstSetListChoices(list, NULL, LstGetSelection(list) + 1);
//        LstDrawList(list);

        if(Side == ComputerSide)
        {
            ComputerBusyMessage();
            Think();
            if(PV[0][0].u != 0)
            {
                move = PV[0][0];
                MakeMove(move);

                location = move.b.to;

                // Clean up remaining move list
//                LstSetListChoices(list, NULL, LstGetSelection(list) + 1);
//                LstDrawList(list);

                // Add new move list
                BrdCoordToWXF(move.b.from, move.b.to, wxf);
                MainAddListItem(list, wxf, true);
                LstSetSelection(list, LstGetNumberOfItems(list) - 1);

                DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);
                DrawPieceBound(form, MainBoardGadget, location, IsFlip);

                Ply = 0;
                Gen();
                ShowGameStatus();
            }
            else
            {
                // Erase busy message
                DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);
            }
        }
    }
}

static Boolean MainOnOpenGame(void)
{
    Boolean handled = false;
    FormType  *form;
    ListType  *list;
    FileHand fileHandle;
    Int16 i;
    char wxf[5];
    Err err;

    form = FrmGetActiveForm();
    list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, MainMoveList));

    fileHandle = FileOpen(0, saveGameName, FileType, CreatorId, fileModeReadOnly, &err);

    if(fileHandle != NULL && err == errNone)
    {
        if(FileError(fileHandle) == 0)
        {
            FileRead(fileHandle, Piece, sizeof(Piece), 1, NULL);
            FileRead(fileHandle, Color, sizeof(Color), 1, NULL);
            FileRead(fileHandle, &Side, sizeof(Side), 1, NULL);
            FileRead(fileHandle, &XSide, sizeof(XSide), 1, NULL);
            FileRead(fileHandle, &Ply, sizeof(Ply), 1, NULL);
            FileRead(fileHandle, &HPly, sizeof(HPly), 1, NULL);
            FileRead(fileHandle, &Fifty, sizeof(Fifty), 1, NULL);
            FileRead(fileHandle, HistDat, sizeof(HistT), HPly, NULL);
            FileRead(fileHandle, &ComputerSide, sizeof(ComputerSide), 1, NULL);
            FileRead(fileHandle, &ComputerDepth, sizeof(ComputerDepth), 1, NULL);
            FileRead(fileHandle, &MaxTime, sizeof(MaxTime), 1, NULL);

            DrawBoard(form, MainBoardGadget, Piece, Color, IsFlip);

            LstSetListChoices(list, NULL, 1);
            LstDrawList(list);

            // Add new move list
            for(i = 0; i < HPly; i++)
            {
                BrdCoordToWXF(HistDat[i].m.b.from, HistDat[i].m.b.to, wxf);
                MainAddListItem(list, wxf, false);
            }
            LstDrawList(list);

            LstSetSelection(list, LstGetNumberOfItems(list) - 1);

            Selected = false;

            Gen();

            FileClose(fileHandle);

            FileDelete(0,saveGameName);

            if(HPly != 0)
            {
                DrawPieceBound(form, MainBoardGadget,
                HistDat[HPly - 1].m.b.to, IsFlip);
            }

            if ( ComputerSide != EMPTY )
                ComputerMakeMove( ComputerSide );

            handled = true;
        }
    }
    else
    {
//        FrmAlert(OpenAlert);
    }
    return handled;
}

#if 0
static Boolean MainOnSaveGame(void)
{
    Boolean handled = false;
    FileHand fileHandle;
    Err err;

    fileHandle = FileOpen(0, "game0", FileType, CreatorId, fileModeReadWrite, NULL);

    if(fileHandle != NULL && err == errNone)
    {
        if(FileError(fileHandle) == 0)
        {
            FileWrite(fileHandle, Piece, sizeof(Piece), 1, NULL);
            FileWrite(fileHandle, Color, sizeof(Color), 1, NULL);
            FileWrite(fileHandle, &Side, sizeof(Side), 1, NULL);
            FileWrite(fileHandle, &XSide, sizeof(XSide), 1, NULL);
            FileWrite(fileHandle, &Ply, sizeof(Ply), 1, NULL);
            FileWrite(fileHandle, &HPly, sizeof(HPly), 1, NULL);
            FileWrite(fileHandle, &Fifty, sizeof(Fifty), 1, NULL);
            FileWrite(fileHandle, HistDat, sizeof(HistT), HPly, NULL);
            FileClose(fileHandle);
            FrmAlert(SaveAlert);
            handled = true;
        }
    }

    return handled;
}
#endif