/***********************************************************************
 *
 * PROJECT: PalmXQ
 * FILE:    util.c
 * AUTHOR:  Cheok Yan Cheng
 *
 * DESCRIPTION: Provide helper function.
 *                   
 * LAST REVISION:   24/02/2002
 *
 ***********************************************************************/

#include <PalmOS.h>
#include <stdarg.h>
#include "palmxqrsc.h"
#include "data.h"
#include "protos.h"
#include "defs.h"

/* check OS version and display warning if not apropiate */
Boolean CheckOSVersion()
{
	Err err;
    UInt32 OSVersion;

	/* get OS version */
	err = FtrGet( sysFtrCreator, sysFtrNumROMVersion, &OSVersion );
	
	/* min OS version check */
	if ( OSVersion < MINOSVERSION || err )
	{
        FrmAlert(OSVersionAlert);
        return false;
	}

    return true; 
}

/* set status with text from resource using printf */
void SetStatus( FormPtr form, UInt16 fieldID, UInt16 id, ... )
{
	va_list args;
	MemHandle strH;
	Char *strP;
    static Char statusStr[STATUS_STR_LEN + 1];
    FieldPtr field;
	
	/* get string from resources */
	strH = DmGetResource( strRsc, id );
	strP = MemHandleLock( strH );
	
	/* build status */
    va_start( args, id );
    StrVPrintF( statusStr, strP, args );
	va_end( args );
	
	/* unlock resource */
	MemHandleUnlock( strH );
	
	/* set string in status */
    field = FrmGetObjectPtr( form, FrmGetObjectIndex( form, fieldID ) );
	FldEraseField( field );
    SetFldText( form, fieldID, statusStr );
	FldDrawField( field );
}


/* chages the text contained in a field */
void SetFldText( FormPtr form, UInt16 fieldID, Char *newStr )
{
	MemHandle strH, oldH;
	Char *strP;
	FieldPtr field;
	
	/* get memory and copy the new string there */
	strH = MemHandleNew( StrLen( newStr ) + 1 );
	strP = MemHandleLock( strH );
	StrCopy( strP, newStr );
	MemHandleUnlock( strH );

	/* get field pointer */
    field = FrmGetObjectPtr( form, FrmGetObjectIndex( form, fieldID ) );

	/* get old handle */
	oldH = FldGetTextHandle( field );
	
	/* assign the new handle to the field */
	FldSetTextHandle( field, strH );

    /* update */
    FldDrawField(field);

	/* free old handle */
	if ( oldH != NULL ) MemHandleFree( oldH );
}

